//
//  SkookumMorseDelegate.m
//
//  Created by Willard Myers on 2008.10.21.
//  Copyright 2008 K1GQ. All rights reserved.
//

#import "CallsignFormatter.h"
#import "ORSSerialPortManager.h"
#import "PreferencesGeneral.h"
#import "SkookumMorseDelegate.h"
#import "WinKeyerConstants.h"
#import "WinKeyerUsb.h"

@interface SkookumMorseDelegate()
- (void)updateSerialPortList;
- (void)serialPortRemoved:(NSNotification*)aNotification;
- (void)serialPortAdded:(NSNotification*)aNotification;
@end

@implementation SkookumMorseDelegate

@synthesize serialPortNameArray;
@synthesize winKeyerUsb;

#pragma mark initialization

- (void) awakeFromNib
{
    [hisCallsignTextField setFormatter:[[CallsignFormatter alloc] init]];
}

- (IBAction)changeSerialPort:(id)sender
{
    NSError* error = nil;
    if (![[WinKeyerUsb sharedInstance] disconnect:&error]) {
        [NSApp presentError:error];
    }
    if (![[[NSUserDefaults standardUserDefaults] stringForKey:kWKPortNamePreferenceKey] isEqualToString:kWKNoPortString]) {
        error = nil;
        if (![[WinKeyerUsb sharedInstance] connect:&error]) {
            [NSApp presentError:error];
        }
    }
}

#pragma mark application delegates

- (void)applicationDidFinishLaunching:(NSNotification*)notification
{
    // Watch for serial port additions and removals.
    [self updateSerialPortList];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(serialPortAdded:)
                                                 name:ORSSerialPortsWereConnectedNotification
                                               object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(serialPortRemoved:)
                                                 name:ORSSerialPortsWereDisconnectedNotification
                                               object:nil];
    
    // Enumerate serial ports and build array of names
    [self updateSerialPortList];

    NSError* error = nil;
    if (![winKeyerUsb connect:&error]) {
        [NSApp presentError:error];
    }
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication*)sender
{
    NSError* error = nil;
    if (![winKeyerUsb disconnect:&error]) {
        [NSApp presentError:error];
    }
    return NSTerminateNow;
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication*)theApplication
{
    return YES;
}

- (void)updateSerialPortList
{
    NSMutableArray* ports = [NSMutableArray array];
    [ports addObject:kWKNoPortString];
    for (ORSSerialPort* port in [ORSSerialPortManager sharedSerialPortManager].availablePorts) {
        [ports addObject:port.name];
	}
    [self setSerialPortNameArray:ports];
}

#pragma mark notification handlers

- (void)serialPortRemoved:(NSNotification*)aNotification
{
    NSLog(@"%s", __PRETTY_FUNCTION__);
    NSDictionary* userInfo = [aNotification userInfo];
    NSArray* removedPorts = userInfo[ORSDisconnectedSerialPortsKey];
    for (id port in removedPorts) {
        NSString* portName = [port name];
        if ([portName isEqualToString:[[NSUserDefaults standardUserDefaults] stringForKey:kWKPortNamePreferenceKey]]) {
            [[NSUserDefaults standardUserDefaults] setObject:kWKNoPortString forKey:kWKNoPortString];
            [[WinKeyerUsb sharedInstance] disconnect:nil];
            NSString* message = [NSString stringWithFormat:@"The WinKeyer using serial port %@ has been unplugged.", [port name]];
            NSAlert* alert = [NSAlert new];
            alert.messageText = message;
            alert.informativeText = @"Plug it back in, or choose another serial port in General preferences.";
            [alert runModal];
            break;
        }
    }
    [self updateSerialPortList];
}

- (void)serialPortAdded:(NSNotification*)aNotification
{
    NSLog(@"%s", __PRETTY_FUNCTION__);
    [self updateSerialPortList];
    // If not connected try using the new device
    if (![[WinKeyerUsb sharedInstance] isConnected]) {
        NSDictionary* userInfo = [aNotification userInfo];
        NSArray* addedPorts = userInfo[ORSConnectedSerialPortsKey];
        for (ORSSerialPort* port in addedPorts) {
            // NSLog(@"added port name: %@", [port name]);
            if ([[port name] hasPrefix:kWKPortNamePrefix]) {
                NSString* message = [NSString stringWithFormat:@"The serial port %@ has been plugged in.", [port name]];
                NSAlert* alert = [NSAlert new];
                alert.messageText = message;
                alert.informativeText = @"Try to use this port to communicate with a WinKeyer?";
                [alert addButtonWithTitle:@"Cancel"];
                NSModalResponse response = [alert runModal];
                if (response == NSModalResponseOK) {
                    [[NSUserDefaults standardUserDefaults] setObject:[port name] forKey:kWKPortNamePreferenceKey];
                    [self changeSerialPort:self];
                }
            }
        }
    }
}

@end

