//
//  WinkeyerController.m
//  SkookumLogger
//
//  Created by Willard Myers on 2009.01.01.
//  Copyright 2009 K1GQ. All rights reserved.
//

#import "AppController.h"
#import "CallFormatter.h"
#import "Constants.h"
#import "KeyboardFieldFormatter.h"
#import "Log.h"
#import "MicroHamDevice.h"
#import "Preferences.h"
#import "RouterCommands.h"
#import "RouterPort.h"
#import "Winkeyer.h"
#import "WinkeyerController.h"
#import "WinkeyerSettingsController.h"
#import <AMSerial/AMSerial.h>

static NSError* sIOError;
static NSCharacterSet * callsignCharacterSet;


@interface WinkeyerController(Private)
- (void) serialPortReadData:(NSDictionary*)dataDictionary;
- (void) serialPortRemoved:(NSNotification*)aNotification;
- (void) serialPortAdded:(NSNotification*)aNotification;
- (void) putToWinkeyer:(NSData*)command;
- (void) sendSerialNumber;
- (NSString*) decodeMessage:(unsigned char*)eepromBuffer start:(NSUInteger)msgStart length:(NSUInteger)msgLength;
- (NSUInteger) encodeMessage:(NSString*)message start:(NSUInteger)start into:(unsigned char*)eeprom;
- (unsigned char)morseToAscii:(const unsigned char)morse;
- (void) spewEeprom:(WinkeyerEEPROM*)eepromBuffer;
- (BOOL) isMicroHamDeviceName:(NSString*)portName;
- (BOOL) openWinkeyer:(NSString*)portName error:(NSError**)error;
- (BOOL) openMicroHamDevice:(NSString*)portName error:(NSError**)error;
@end

@implementation WinkeyerController

@synthesize connectButtonTitle;
@synthesize connected;
@synthesize isWinKeyer2;
@synthesize keyboardBuffer;
@synthesize nextKeyboardBufferCharacterIndex;
@synthesize sendingKeyboardBuffer;
@synthesize speedString;
@synthesize standaloneMessage1;
@synthesize standaloneMessage2;
@synthesize standaloneMessage3;
@synthesize standaloneMessage4;
@synthesize standaloneMessage5;
@synthesize standaloneMessage6;
@synthesize versionString;
@synthesize winkeyerPort;

#pragma mark defaults


#pragma mark accessors

#pragma mark public methods

- (NSString*) portName
{
    return [[self winkeyerPort] name];
}

// Return YES if successfully opened
- (BOOL) open:(NSError**)error
{
    BOOL didOpen = NO;
    NSString* portName = [[NSUserDefaults standardUserDefaults] objectForKey:kWKPortName];
    if ([self isMicroHamDeviceName:portName]) {
        didOpen = [self openMicroHamDevice:portName error:error];
    } else {
        didOpen = [self openWinkeyer:portName error:error];
    }
    if (didOpen) {
        [self loadWinkeyerDefaults];        
        // Tickle winkeyer to cause update of speed display
        [self putToWinkeyer:byteToData(kWKGetSpeedPot)];                    
        [self setConnectButtonTitle:@"Disconnect"];
        [self setConnected:YES];
    }        
    return didOpen;
}

- (void) close
{
    //    NSLog(@"close");
    [self abort:self];
    if (microHamDevice != nil) {
        [microHamDevice writeWinkey:bytesToData(kWKAdminClose, 2)];
        [microHamDevice close];
        microHamDevice = nil;
    } else if (winkeyerPort != nil) {
        [[self winkeyerPort] stopReadInBackground];
        // Force winkeyer to output something so the read thread notices
        // that it should stop.
        [self putToWinkeyer:byteToData(kWKRequestStatus)];
        // Now close the winkeyer; this *should* put it back in
        // standalone mode, but doesn't ???
        [self putToWinkeyer:bytesToData(kWKAdminClose, 2)];
        [[self winkeyerPort] free]; // sends itself a close message
        [self setWinkeyerPort:nil];
    }
    wkEEPROM = nil;
    [self setVersionString:nil];
    [self setConnectButtonTitle:@"Connect"];
    [self setConnected:NO];
}

- (void) send
{
}

- (void) toggleConnection
{
//    NSLog(@"winkeyerController.connect");
    NSString* portName = [[NSUserDefaults standardUserDefaults] objectForKey:kWKPortName];
    if ([portName compare:kPortName_None] == NSOrderedSame) return;
    if ([[self connectButtonTitle] isEqualToString:@"Connect"]) {
        NSError* error = [[NSError alloc] init];
        if (![self open:&error]) {
            NSAlert* openAlert = [NSAlert alertWithError:error];
            [openAlert runModal];
            [self close];
        }
    } else {
        [self close];
    }
}

- (void) readStandalone
{
    NSLog(@"WinkeyerController:readStandalone");
    if (![self connected]) {
        NSLog(@"Programming error, readStandalone called with no winkeyer connection.");
        return;
    }

    [[self winkeyerPort] setReadTimeout:10.0];
    NSError* error = [[NSError alloc] init];
    if (![[self winkeyerPort] writeData:bytesToData(kWKAdminDumpEEPROM, 2) error:&error]) {
        NSAlert* alert = [NSAlert alertWithError:error];
        [alert runModal];
    }
    wkEEPROM = nil;
    wkEEPROM = [[self winkeyerPort] readAndReturnError:&error];
    if (wkEEPROM) {
        if ([wkEEPROM length] != 256) {
            NSLog(@"Dump EEPROM returned %d bytes\n %@", [wkEEPROM length], wkEEPROM);
            return;
        }
        WinkeyerEEPROM* eepromBuffer = (WinkeyerEEPROM*)[wkEEPROM bytes];
        [self spewEeprom:eepromBuffer];
    } else {
        NSAlert* alert = [NSAlert alertWithError:error];
        [alert runModal];
        return;
    }
}

- (void) spewEeprom:(WinkeyerEEPROM*)eepromBuffer
{
    NSLog(@"Magic = %x", eepromBuffer->magic);
    unsigned char wk2mode = eepromBuffer->modereg;
    if (wk2mode & 0x80) NSLog(@"Paddle watchdog disabled");
    else                NSLog(@"Paddle watchdog enabled");
    if (wk2mode & 0x40) NSLog(@"Paddle echoback enabled");
    else                NSLog(@"Paddle echoback disabled");
    if ((wk2mode & 0x20) && (wk2mode & 0x10))       NSLog(@"Bug mode");
    else if ((wk2mode & 0x20) && !(wk2mode & 0x10)) NSLog(@"Ultimatic mode");
    else if (!(wk2mode & 0x20) && (wk2mode & 0x10)) NSLog(@"Iambic A mode");
    else if (!(wk2mode & 0x20) && !(wk2mode & 0x10)) NSLog(@"Iambic B mode");
    if (wk2mode & 0x08) NSLog(@"Paddles swapped");
    else                NSLog(@"Paddles not swapped");
    if (wk2mode & 0x04) NSLog(@"Serial echoback enabled");
    else                NSLog(@"Serial echoback disabled");
    if (wk2mode & 0x02) NSLog(@"Autospace enabled");
    else                NSLog(@"Autospace disabled");
    if (wk2mode & 0x01) NSLog(@"CT word spacing");
    else                NSLog(@"Normal word spacing");
    NSLog(@"Speed = %d", eepromBuffer->oprrate);
    unsigned char sidetone = eepromBuffer->stconst;
    if (sidetone & 0x80) NSLog(@"Paddle only sidetone enabled");
    else                 NSLog(@"Paddle only sidetone disabled");
    switch (sidetone & 0x0F) {
        case 0x01: NSLog(@"Sidetone frequency = 4000 Hz"); break;
        case 0x02: NSLog(@"Sidetone frequency = 2000 Hz"); break;
        case 0x03: NSLog(@"Sidetone frequency = 1333 Hz"); break;
        case 0x04: NSLog(@"Sidetone frequency = 1000 Hz"); break;
        case 0x05: NSLog(@"Sidetone frequency = 800 Hz"); break;
        case 0x06: NSLog(@"Sidetone frequency = 666 Hz"); break;
        case 0x07: NSLog(@"Sidetone frequency = 571 Hz"); break;
        case 0x08: NSLog(@"Sidetone frequency = 500 Hz"); break;
        case 0x09: NSLog(@"Sidetone frequency = 444 Hz"); break;
        case 0x0A: NSLog(@"Sidetone frequency = 400 Hz"); break;
    }
    NSLog(@"Weight = %d", eepromBuffer->weight);
    NSLog(@"PTT lead time = %d", eepromBuffer->lead_time);
    NSLog(@"PTT tail time = %d", eepromBuffer->tail_time);
    NSLog(@"Minimum Speed = %d", eepromBuffer->minwpm);
    NSLog(@"Speed range = %d", eepromBuffer->wpmrange);
    NSLog(@"First element exension = %d", eepromBuffer->xtnd);
    NSLog(@"Keying compensation = %d", eepromBuffer->kcomp);
    NSLog(@"Farnsworth speed = %d", eepromBuffer->farnswpm);
    NSLog(@"Paddle setpoint = %d", eepromBuffer->sampadj);
    NSLog(@"Ratio = %d", eepromBuffer->ratio);
    unsigned char pinConfig = eepromBuffer->pincfg;
    if ((pinConfig & 0x08) && (pinConfig & 0x40))       NSLog(@"Undefined Ultimatic priority");
    else if ((pinConfig & 0x80) && !(pinConfig & 0x40)) NSLog(@"Both sends dits");
    else if (!(pinConfig & 0x80) && (pinConfig & 0x40)) NSLog(@"Both sends dahs");
    else                                                NSLog(@"Both alternates");
    if ((pinConfig & 0x20) && (pinConfig & 0x10))       NSLog(@"Paddle hang time = 2.0 * letterspace");
    else if ((pinConfig & 0x20) && !(pinConfig & 0x10)) NSLog(@"Paddle hang time = 1.66 * letterspace");
    else if (!(pinConfig & 0x20) && (pinConfig & 0x10)) NSLog(@"Paddle hang time = 1.33 * letterspace");
    else                                                NSLog(@"Paddle hang time = 1.0 * letterspace");
    if (pinConfig & 0x08) NSLog(@"Keyout 2 enabled");
    else                  NSLog(@"Keyout 2 disabled");
    if (pinConfig & 0x04) NSLog(@"Keyout 1 enabled");
    else                  NSLog(@"Keyout 1 disabled");
    if (pinConfig & 0x02) NSLog(@"Sidetone enabled");
    else                  NSLog(@"Sidetone disabled");
    if (pinConfig & 0x01) NSLog(@"PTT enabled");
    else                  NSLog(@"PTT disabled");
    unsigned char k12mode = eepromBuffer->k12mode;
    if (k12mode & 0x04) NSLog(@"Message break by paddle enabled");
    else                NSLog(@"Message break by paddle disabled");
    if (k12mode & 0x08) NSLog(@"Cut serial numbers enabled");
    else                NSLog(@"Cut serial numbers disabled");
    NSLog(@"Command WPM = %d", eepromBuffer->cmdwpm);
    
    NSLog(@"Free pointer = %d", eepromBuffer->freeptr);
    for (NSUInteger i = 0; i < 6; ++i) {
        NSInteger msgStart = eepromBuffer->msgptrs[i] - WK_MESSAGES_BUFFER_OFFSET;
        NSInteger msgLength = 0;
        if (i == 5) {
            msgLength = eepromBuffer->freeptr - eepromBuffer->msgptrs[i];
        } else {
            msgLength = eepromBuffer->msgptrs[i+1] - eepromBuffer->msgptrs[i];
        }
        NSString* msg = [self decodeMessage:eepromBuffer->msgs start:msgStart length:msgLength];
        NSLog(@"msg %d ptr %d start = %d  length = %d  content = %@", i, eepromBuffer->msgptrs[i], msgStart, msgLength, msg);
        switch (i) {
            case 0: [self setStandaloneMessage1:msg]; break;
            case 1: [self setStandaloneMessage2:msg]; break;
            case 2: [self setStandaloneMessage3:msg]; break;
            case 3: [self setStandaloneMessage4:msg]; break;
            case 4: [self setStandaloneMessage5:msg]; break;
            case 5: [self setStandaloneMessage6:msg]; break;
        }
    }
}

- (void) writeStandalone
{
    NSLog(@"WinkeyerController:writeStandalone");
    WinkeyerEEPROM* eeprom = calloc(sizeof(WinkeyerEEPROM), sizeof(unsigned char));    
    eeprom->magic = 0xA5;
    NSUserDefaults* sd = [NSUserDefaults standardUserDefaults];
    if ([sd boolForKey:kWKDisablePaddleWatchdog]) eeprom->modereg |= 0x80;
    if ([sd boolForKey:kWKPaddleEchoback]) eeprom->modereg |= 0x40;
    NSString* keyMode = [sd stringForKey:kWKKeyMode];
    if ([keyMode isEqualToString:kWKKeyModeBug]) {
        eeprom->modereg |= 0x30;
    } else if ([keyMode isEqualToString:kWKKeyModeUltimatic]) {
        eeprom->modereg |= 0x20;
    } else if ([keyMode isEqualToString:kWKKeyModeIambicA]) {
        eeprom->modereg |= 0x10;
    }
    if ([sd boolForKey:kWKPaddleSwap]) eeprom->modereg |= 0x08;
    if ([sd boolForKey:kWKSerialEchoback]) eeprom->modereg |= 0x04;
    if ([sd boolForKey:kWKAutospace]) eeprom->modereg |= 0x02;
    if ([sd boolForKey:kWKContestSpacing]) eeprom->modereg |= 0x01;
    if ([sd boolForKey:kWKPaddleOnlySidetone]) eeprom->stconst |= 0x80;
    if ([[sd stringForKey:kWKSidetoneFrequency] isEqualToString:kWKSidetone4000]) eeprom->stconst |= 0x01;
    if ([[sd stringForKey:kWKSidetoneFrequency] isEqualToString:kWKSidetone2000]) eeprom->stconst |= 0x02;
    if ([[sd stringForKey:kWKSidetoneFrequency] isEqualToString:kWKSidetone1333]) eeprom->stconst |= 0x03;
    if ([[sd stringForKey:kWKSidetoneFrequency] isEqualToString:kWKSidetone1000]) eeprom->stconst |= 0x04;
    if ([[sd stringForKey:kWKSidetoneFrequency] isEqualToString:kWKSidetone800]) eeprom->stconst |= 0x05;
    if ([[sd stringForKey:kWKSidetoneFrequency] isEqualToString:kWKSidetone666]) eeprom->stconst |= 0x06;
    if ([[sd stringForKey:kWKSidetoneFrequency] isEqualToString:kWKSidetone571]) eeprom->stconst |= 0x07;
    if ([[sd stringForKey:kWKSidetoneFrequency] isEqualToString:kWKSidetone500]) eeprom->stconst |= 0x08;
    if ([[sd stringForKey:kWKSidetoneFrequency] isEqualToString:kWKSidetone444]) eeprom->stconst |= 0x09;
    if ([[sd stringForKey:kWKSidetoneFrequency] isEqualToString:kWKSidetone400]) eeprom->stconst |= 0x0A;
    eeprom->weight = [[sd objectForKey:kWKWeighting] unsignedCharValue];
    eeprom->lead_time = [[sd objectForKey:kWKPTTLeadTime] unsignedCharValue];
    eeprom->tail_time = [[sd objectForKey:kWKPTTTailTime] unsignedCharValue];
    eeprom->minwpm = [[sd objectForKey:kWKSpeedPotMinimum] unsignedCharValue];
    eeprom->wpmrange = [[sd objectForKey:kWKSpeedPotRange] unsignedCharValue];
    eeprom->xtnd = [[sd objectForKey:kWKFirstExtension] unsignedCharValue];
    eeprom->kcomp = [[sd objectForKey:kWKKeyingCompensation] unsignedCharValue];
    eeprom->farnswpm = [[sd objectForKey:kWKFarnsworthSpeed] unsignedCharValue];
    eeprom->sampadj = [[sd objectForKey:kWKPaddleSwitchpoint] unsignedCharValue];
    eeprom->ratio = [[sd objectForKey:kWKRatio] unsignedCharValue];
    if ([[sd stringForKey:kWKDitDahPriority] isEqualToString:kWKPriorityDah]) eeprom->pincfg |= 0x40;
    if ([[sd stringForKey:kWKDitDahPriority] isEqualToString:kWKPriorityDit]) eeprom->pincfg |= 0x80;
    if ([[sd stringForKey:kWKPaddleHang] isEqualToString:kWKPaddleHang1_33]) eeprom->pincfg |= 0x10;
    if ([[sd stringForKey:kWKPaddleHang] isEqualToString:kWKPaddleHang1_66]) eeprom->pincfg |= 0x20;
    if ([[sd stringForKey:kWKPaddleHang] isEqualToString:kWKPaddleHang2]) eeprom->pincfg |= 0x30;
    if ([sd boolForKey:kWKKeyOut2Enable]) eeprom->pincfg |= 0x08;
    if ([sd boolForKey:kWKKeyOut1Enable]) eeprom->pincfg |= 0x04;
    if ([sd boolForKey:kWKSidetoneEnable]) eeprom->pincfg |= 0x02;
    if ([sd boolForKey:kWKPTTEnable]) eeprom->pincfg |= 0x01;
    if ([sd boolForKey:kWKEnableMessageBreak]) eeprom->k12mode |= 0x04;
    if ([sd boolForKey:kWKCutSerialNumbers]) eeprom->k12mode |= 0x08;
    eeprom->cmdwpm = [[sd objectForKey:kWKCommandSpeed] unsignedCharValue];
    
    NSUInteger msgStart = 0;
    for (NSUInteger msgNumber = 1; msgNumber < 7; ++msgNumber) {
        NSString* key = [NSString stringWithFormat:@"WKMessage%d", msgNumber];
        NSString* message = [[sd objectForKey:key] uppercaseString];
        NSUInteger end = [self encodeMessage:message start:msgStart into:eeprom->msgs];
        eeprom->msgptrs[msgNumber - 1] = msgStart + WK_MESSAGES_BUFFER_OFFSET;
        msgStart += end;
    }
    eeprom->freeptr = msgStart + WK_MESSAGES_BUFFER_OFFSET;

    // Clobber whatever is going on
    [self abort:self];
    [[self winkeyerPort] stopReadInBackground];
    // Force winkeyer to output something so the read thread notices
    // that it should stop.
    [self putToWinkeyer:byteToData(kWKRequestStatus)];
    
    // Must be in ADMIN  mode to write eeprom   
    NSError* error = [[NSError alloc] init];
    [[self winkeyerPort] writeData:bytesToData(kWKAdminClose, 2) error:&error];
    [[self winkeyerPort] writeData:bytesToData(kWKAdminLoadEEPROM, 2) error:&error];
    if (![[self winkeyerPort] writeData:bytesToData((unsigned char*)eeprom, 256) error:&error]) {
        NSAlert* alert = [NSAlert alertWithError:error];
        [alert runModal];
    }
    
    [self readStandalone];

    // Return to your normal program
    [[self winkeyerPort] writeData:bytesToData(kWKAdminOpen, 2) error:&error];
    [[self winkeyerPort] readDataInBackground];
    // Tickle winkeyer to cause update of speed display
    [self putToWinkeyer:byteToData(kWKGetSpeedPot)];    
    
    free(eeprom);
}

#pragma mark actions

- (IBAction) abort:(id)sender
{
    //    NSLog(@"abort");
    [self putToWinkeyer:byteToData(kWKClearBuffer)];

    id doc = [[NSDocumentController sharedDocumentController] currentDocument];
    
    [doc setSendingHisCall:NO];
    [doc setNextHisCallCharacterIndex:0];
    
    [doc setSendingMyCall:NO];
    [doc setNextMyCallCharacterIndex:0];
    
    [doc setSendingMessage:NO];
    [doc setNextMessageCharacterIndex:0];
    [doc setMessage:nil];
    [[doc messageNumberFifo] removeAllObjects];
    
    [self setSendingKeyboardBuffer:NO];
    [self setKeyboardBuffer:@""];
    [self setNextKeyboardBufferCharacterIndex:0];
    
}

- (IBAction) pause:(id)sender
{
    unsigned char command[2] = {kWKSetPauseState, 0x00};
    if ([sender state] == NSOnState) {
        command[1] = 0x01;
    }
    [self putToWinkeyer:bytesToData(command, 2)];
}

#pragma mark private methods

- (BOOL) isMicroHamDeviceName:(NSString*)portName
{
    return ([portName hasPrefix:@"usbserial-MK"] || [portName hasPrefix:@"usbserial-CK"]);
}

- (BOOL) openMicroHamDevice:(NSString*)portName error:(NSError**)error
{
    BOOL didOpenDevice = NO;
    if ([[NSApp delegate] routerDidLaunch]) {
        if ([portName hasPrefix:@"usbserial-MK"]) {
            // Try to open microKEYER
            int routerRead = open("/tmp/microHamRouterRead", O_RDONLY);
            int routerWrite = open("/tmp/microHamRouterWrite", O_WRONLY);
            int keyerRead;
            int keyerWrite;
            obtainRouterPorts(&keyerRead, &keyerWrite, OPENMICROKEYER, routerRead, routerWrite);
            if (keyerRead > 0 && keyerWrite > 0) {
                microHamDevice = [[MicroHamDevice alloc] initWithReadPipe:keyerRead writePipe:keyerWrite delegate:self];
            }
            close(routerRead);
            close(routerWrite);
        } else if ([portName hasPrefix:@"usbserial-CK"]) {
            // Try to open CW Keyer
            int routerRead = open("/tmp/microHamRouterRead", O_RDONLY);
            int routerWrite = open("/tmp/microHamRouterWrite", O_WRONLY);
            int keyerRead;
            int keyerWrite;
            obtainRouterPorts(&keyerRead, &keyerWrite, OPENCWKEYER, routerRead, routerWrite);
            if (keyerRead > 0 && keyerWrite > 0) {
                microHamDevice = [[MicroHamDevice alloc] initWithReadPipe:keyerRead writePipe:keyerWrite delegate:self];
            }
            close(routerRead);
            close(routerWrite);
        }
    } else {
        if (error) {
            NSString* errorDescription = [NSString stringWithFormat:@"µH Router is not running."];
            NSString* errorDetail = @"Confirm that the µH Router application is properly installed on your computer.";
            NSDictionary* errorDict = [NSDictionary dictionaryWithObjectsAndKeys:errorDescription, NSLocalizedDescriptionKey, errorDetail, NSLocalizedRecoverySuggestionErrorKey, nil];
            *error = [NSError errorWithDomain:@"comk1gqskookumlogger" code:1 userInfo:errorDict];
        }
    }
    
    if ([microHamDevice winkeyDidOpen]) {
        winkeyResponse = nil;
        // Ugly hack to minimize delay waiting for response to AdminOpen command
        [microHamDevice writeWinkey:bytesToData(kWKAdminOpen, 2)];
        useconds_t dt = 500000;
        for (NSUInteger i = 0; i < 10; ++i) {
            usleep(dt);
            if (winkeyResponse != nil) break;
        }
        if (winkeyResponse == nil) {
            if (error) {
                NSString* errorDescription = [NSString stringWithFormat:@"The microHAM winkey did not respond to the host open command.", portName];
                NSString* errorDetail = @"Confirm that the microHAM device is seen by µH Router, and that it has DC power.";
                NSDictionary* errorDict = [NSDictionary dictionaryWithObjectsAndKeys:errorDescription, NSLocalizedDescriptionKey, errorDetail, NSLocalizedRecoverySuggestionErrorKey, nil];
                *error = [NSError errorWithDomain:@"comk1gqskookumlogger" code:1 userInfo:errorDict];
            }
        } else {
            didOpenDevice = YES;
            unsigned char version = ((unsigned char*)[winkeyResponse bytes])[0];
            [self setVersionString:[NSString stringWithFormat:@"%d", version]];
            [self setIsWinKeyer2:(version >= 20)];
        }
    } else {
        if (error) {
            NSString* errorDescription = [NSString stringWithFormat:@"µH Router failed to provide a connection to the microHAM device %@.", portName];
            NSString* errorDetail = @"Confirm that the microHAM device is seen by µH Router.";
            NSDictionary* errorDict = [NSDictionary dictionaryWithObjectsAndKeys:errorDescription, NSLocalizedDescriptionKey, errorDetail, NSLocalizedRecoverySuggestionErrorKey, nil];
            *error = [NSError errorWithDomain:@"comk1gqskookumlogger" code:1 userInfo:errorDict];
        }
    }
    return didOpenDevice;
}

- (BOOL) openWinkeyer:(NSString*)portName error:(NSError**)error
{
}

- (void)serialPortReadData:(NSDictionary*)dataDictionary
{
    // Called when data arrives from Winkeyer USB
    NSData* input = [dataDictionary objectForKey:@"data"];
    // NSLog(@"serialPortReadData with data: %@", input);
    if ([input length] == 0) {
        NSLog(@"Winkeyer port was closed!");
        return;
    }
    [self handleInputFromWinkey:input];
    // Continue listening
    [[self winkeyerPort] readDataInBackground];
}

- (void) handleInputFromWinkey:(NSData*)input
{
//    NSLog(@"Input from Winkey: %@", input);
    winkeyResponse = input;
    id doc = [[NSDocumentController sharedDocumentController] currentDocument];
    for (NSUInteger i = 0; i < [input length]; ++i) {
        unsigned char aByte = ((unsigned char*)[input bytes])[0];
        if (aByte == 0xFF) {
            NSLog(@"error byte");
        } else if ((aByte & 0xC0) == 0xC0) {
            // NSLog(@"Status byte: %x", aByte);
            bool paddleBreakin = (aByte & 0x02);
            if (paddleBreakin) {
                // NSLog(@"Sending aborted by paddle");
                if ([doc sendingMessage] || [doc sendingHisCall] || [self sendingKeyboardBuffer]) {
                    [self abort:nil];
                }
            } else {
                [self send];
            }
        } else if ((aByte & 0xC0) == 0x80) {
            // NSLog(@"Speed pot changed byte: %x", aByte);
            NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
            NSInteger speedPotMinimum = [defaults integerForKey:kWKSpeedPotMinimum];
            // Get Speed Pot response
            NSInteger response = aByte & 0x3F;
            NSInteger potSpeedWPM = response + speedPotMinimum;
            [self setSpeedString:[NSString stringWithFormat:@"%d wpm", potSpeedWPM]];
        } else {
            // NSLog(@"Echo character: %x", aByte);
            // Assume it's the expected character
            [self send];
        }
    }
}

- (NSString*) decodeMessage:(unsigned char*)messages start:(NSUInteger)msgStart length:(NSUInteger)msgLength
{
    unsigned char* sbuf = calloc(msgLength, sizeof(unsigned char));
    for (NSUInteger i = 0; i < msgLength; ++i) {
        sbuf[i] = messages[msgStart + i];
        if (sbuf[i] & 0x80) {
            sbuf[i] = [self morseToAscii:(sbuf[i] & 0x7f)];
            break;
        } else {
            sbuf[i] = [self morseToAscii:(sbuf[i])];
        }
    }
    NSString* message = [[NSString alloc] initWithCString:(char*)sbuf length:msgLength];
    free(sbuf);
    return message;
}

- (unsigned char)morseToAscii:(const unsigned char)morse
{
    for (NSUInteger i = 0; i < 96; ++i) {
        if (ASCII_TO_MORSE[i] == morse) {
            return i + 0x20;
        }
    }
    return 0;
}

@end
